@@ -10,17 +10,16 @@ import attrdict
 from getpass import getpass
 
 
-
+def botlog(message, error = False):
+    ERROR = "\033[1;31;40m"
+    NORMAL = "\033[m"
+    print((ERROR if error else NORMAL) + message + NORMAL)
+    sys.stdout.flush()
 
 class Bot:
 
-    def log(self, message, error = False):
-        ERROR = "\033[1;31;40m"
-        NORMAL = "\033[m"
-        print((ERROR if error else NORMAL) + message)
-        sys.stdout.flush()
 
-    def __init__(self, nick, room = "bots", owner = None, password = "", help = None, ping = None, important = False, killed = None, API_timeout = 10, advanced = False, function = None):
+    def __init__(self, nick, room = "bots", owner = None, password = "", help = None, ping = None, important = False, killed = None, API_timeout = 10, advanced = False, function = None, sendable = False):
         parser = argparse.ArgumentParser(description=f'{nick}: A euphoria.io bot.')
         parser.add_argument("--test", "--debug", "-t", help = "Used to debug dev builds. Sends bot to &test instead of its default room.", action = 'store_true')
         parser.add_argument("--room", "-r", help = f"Set the room the bot will be placed in. Default: {room}", action = "store", default = room)
@@ -30,7 +29,7 @@ class Bot:
 
         self.nick = nick
         self.room = args.room if args.test != True else "test"
-        self.log("Debug: " + str(args.test))
+        botlog("Debug: " + str(args.test))
         self.normname = re.sub(r"\s+", "", self.nick)
         self.owner = owner if owner != None else "nobody"
         self.normowner = re.sub(r"\s+", "", self.owner)
@@ -43,6 +42,7 @@ class Bot:
         self.API_timeout = API_timeout
         self.advanced = advanced
         self.function = function
+        self.sendable = sendable
 
 
     def connect(self):
@@ -57,50 +57,55 @@ class Bot:
             self._handle_auth(self.password)
             self.set_nick(self.nick)
         else:
-            self.log(reply, error = True)
-        self.log(f'connected to &{self.room}.')
+            botlog(reply, error = True)
+        botlog(f'connected to &{self.room}.')
 
     def send_msg(self, msgString, parent = None):
         if re.search(r"^\[.+,.+\]$", msgString):
             msgString = random.choice(msgString[1:-1].split(","))
-        if isinstance(parent, AttrDict):
-            self.conn.send(json.dumps({'type': 'send', 'data': {'content': msgString, 'parent': parent.data.id}}))
-            reply = None
-            i = 0
-            for msgJSON in self.conn:
-                i += 1
-                msg = AttrDict(json.loads(msgJSON))
-                if msg.type == "send-reply":
-                    reply = msg
-                    self.log(f'Message sent: {reply} replying to: {parent.data.id} by {parent.data.sender.name}')
-                    return reply
-                if i > self.API_timeout:
-                    self.log("send-reply API response not recorded. reason: too many events before send-reply.", True)
-                    return None
+        try:
+            if isinstance(parent, AttrDict):
+                self.conn.send(json.dumps({'type': 'send', 'data': {'content': msgString, 'parent': parent.data.id}}))
+                reply = None
+                i = 0
+                for msgJSON in self.conn:
+                    i += 1
+                    msg = AttrDict(json.loads(msgJSON))
+                    if msg.type == "send-reply":
+                        reply = msg
+                        botlog(f'Message sent: {reply} replying to: {parent.data.id} by {parent.data.sender.name}')
+                        return reply
+                    if i > self.API_timeout:
+                        botlog("send-reply API response not recorded. reason: too many events before send-reply.", True)
+                        return None
 
-        elif isinstance(parent, str):
-            self.conn.send(json.dumps({'type': 'send', 'data': {'content': msgString, 'parent': parent}}))
-            reply = None
-            i = 0
-            for msgJSON in self.conn:
-                i += 1
-                msg = AttrDict(json.loads(msgJSON))
-                if msg.type == "send-reply":
-                    reply = msg
-                    self.log(f'Message sent: {reply} replying to: {parent}')
-                    return reply
-                if i > self.API_timeout:
-                    self.log("send-reply API response not recorded. reason: too many events before send-reply.", True)
-                    return None
+            elif isinstance(parent, str):
+                self.conn.send(json.dumps({'type': 'send', 'data': {'content': msgString, 'parent': parent}}))
+                reply = None
+                i = 0
+                for msgJSON in self.conn:
+                    i += 1
+                    msg = AttrDict(json.loads(msgJSON))
+                    if msg.type == "send-reply":
+                        reply = msg
+                        botlog(f'Message sent: {reply} replying to: {parent}')
+                        return reply
+                    if i > self.API_timeout:
+                        botlog("send-reply API response not recorded. reason: too many events before send-reply.", True)
+                        return None
 
-        else:
-            raise BadMessageError(f'message not sent, because type of parent was {type(parent)}').with_traceback(f'parent = {parent}', error = True)
+            else:
+                raise BadMessageError(f'message not sent, because type of parent was {type(parent)}. \nParent printed: \n{parent}')
+        except BadMessageError:
+            pass
 
 
 
 
     def restart(self, msg = None):
         self.conn.close()
+        del self.conn
+        botlog("restarting...")
         self.connect()
         self.start()
         self.send_msg("Restarted", parent = msg)
@@ -118,7 +123,7 @@ class Bot:
                     elif msg.type == 'send-event' and msg.data.sender.name != self.nick:
                         self._handle_message(msg)
                     elif msg.type == 'error':
-                        self.log(msg, error = True)
+                        botlog(msg, error = True)
                     elif msg.type == 'bounce-event':
                         self._handle_auth(msg)
                     else:
@@ -129,7 +134,7 @@ class Bot:
             if self.function == None and function != None and callable(function):
                 self.function = function
             else:
-                self.log("No function passed to advanced bot's start function.")
+                botlog("No function passed to advanced bot's start function.")
             try:
                 for msgJSON in self.conn:
                     msg = AttrDict(json.loads(msgJSON))
@@ -139,7 +144,7 @@ class Bot:
                         if re.search(f'^!forcekill @{self.normname}$', msg.data.content) != None:
                             self.kill()
                         if re.search(f'^!restart @{self.normname}$', msg.data.content) != None:
-                            self.restart()
+                            self.restart(msg = msg)
                     if msg.type in self.handlers.keys():
                         self.handlers[msg.type](msg)
                     else:
@@ -148,7 +153,7 @@ class Bot:
                         elif callable(self.function):
                             self.function(msg)
                         else:
-                            self.log("Advanced start must be given a callable message handler function that takes an AttrDict as its argument.", error = True)
+                            botlog("Advanced start must be given a callable message handler function that takes an AttrDict as its argument.", error = True)
             except KilledError:
                 pass
 
@@ -194,6 +199,8 @@ class Bot:
             self.send_msg(self.ping, parent = msg)
         if re.search(f'^!help @(?i){self.normname}$', msg.data.content) != None:
             self.send_msg(self.help, parent = msg)
+        if re.search(f'^!send @(?i){self.normname} &(\w+)$', msg.data.content) != None and self.sendable:
+            self.move_to(re.search(f'^!send @(?i){self.normname} &(\w+)$', msg.data.content).group(0))
         for regex, response in self._regexes.items():
             if re.search(regex, msg.data.content) != None:
                 if callable(response):
@@ -229,13 +236,13 @@ class Bot:
             if msg.type == "auth-reply":
                 reply = msg
                 if reply.data.success == True:
-                    self.log(f'Successfully logged into {self.room}.')
+                    botlog(f'Successfully logged into {self.room}.')
                 else:
-                    self.log(f'Login unsuccessful. Reason: {reply.data.reason}', error = True)
+                    botlog(f'Login unsuccessful. Reason: {reply.data.reason}', error = True)
                     self._handle_auth(getpass("Enter the correct password: "))
                 break
             if i > self.API_timeout:
-                self.log("Auth Error: auth-reply API response not recorded.", error = True)
+                botlog("Auth Error: auth-reply API response not recorded.", error = True)
                 break
 
     def _handle_other(self, msg):
@@ -261,7 +268,7 @@ class Bot:
                 reply = msg
                 return reply.data.listing
             if i > self.API_timeout:
-                self.log("who-reply API response not recorded. reason: too many events before who-reply.", error = True)
+                botlog("who-reply API response not recorded. reason: too many events before who-reply.", error = True)
                 return None
 
 
@@ -274,7 +281,7 @@ class Bot:
         if callable(function):
             self.handlers += {eventString : function}
         else:
-            self.log(f"WARNING: handler for {eventString} not callable, handler not set.", error = True)
+            botlog(f"WARNING: handler for {eventString} not callable, handler not set.", error = True)
 
 class BotError(Exception):
     pass
@@ -282,9 +289,9 @@ class BotError(Exception):
 class KilledError(BotError):
     def __init__(self, message = "bot killed."):
         self.message = message
-        self.log(f'KilledError: {message}', error = True)
+        botlog(f'KilledError: {message}', error = True)
 
 class BadMessageError(BotError):
     def __init__(self, message):
         self.message = message
-        self.log(f'BadMessageError: {message}', error = True)
+        botlog(f'BadMessageError: {message}', error = True)
